/* 
    This file si part of CellShaper.
    
    CellShaper is a pack of ImageJ macros developed 
    to analyze the morphology of hippocampal stem cells,
    Copyright (C) 2019  Jorge Valero Gómez-Lobo.

   CellShaper is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

   CellShaper is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <https://www.gnu.org/licenses/>.
*/

//This macro has been developed by Dr Jorge Valero (jorge.valero@achucarro.org). 
//If you have any doubt about how to use it, please contact me.

//License
Dialog.create("GNU GPL License");
Dialog.addMessage("CellShaper (C) 2019 Jorge Valero Gomez-Lobo.");
Dialog.setInsets(10, 20, 0);
Dialog.addMessage("CellShaper comes with ABSOLUTELY NO WARRANTY; click on help button for details.");
Dialog.setInsets(0, 20, 0);
Dialog.addMessage("This is free software, and you are welcome to redistribute it under certain conditions; click on help button for details.");
Dialog.addHelp("https://www.gnu.org/licenses/");
Dialog.show();

//Get info of the folders
dir=getDirectory("Please, select the general directory");
Folders=getFileList(dir);

//Reset images and RoiManager
run("Close All");
roiManager("reset");

//Read and create folders
dirIm=dir+"M1 Images/";
dirNotUsed=dir+"M1 Not used/";
dirProcessed=dir+"M1 Processed images/";
dirROIs=dir+"M1 ROIs/";
if (File.exists(dirNotUsed)==false) File.makeDirectory(dirNotUsed);
if (File.exists(dirProcessed)==false) File.makeDirectory(dirProcessed);
if (File.exists(dirROIs)==false) File.makeDirectory(dirROIs);

//Read folders to load images and proceed them
Imagesfolders=getFileList(dirIm);
for (i=0; i<Imagesfolders.length; i++){
	//Read Images folder
	if (endsWith(Imagesfolders[i], "/")){
		  //Get group folder and name
		 sizefoldName= lengthOf(Imagesfolders[i]);
		 groupName=substring(Imagesfolders[i], 0, sizefoldName-1);
		 Groupfolders=getFileList(dirIm+Imagesfolders[i]);
		 //Read Group folders
		 for (ii=0; ii<Groupfolders.length; ii++){
		 	//Get experiment folder and name
		 	if (endsWith(Groupfolders[ii], "/")){
				sizefoldName= lengthOf(Groupfolders[ii]);
		 		expName=substring(Groupfolders[ii], 0, sizefoldName-1);
				 ImageFiles=getFileList(dirIm+Imagesfolders[i]+Groupfolders[ii]);
				//Read ROIs folder
				 for (iii=0; iii<ImageFiles.length; iii++){
				 	sizefoldName= lengthOf(ImageFiles[iii]);
				 	ImaName=substring(ImageFiles[iii], 0, sizefoldName-4);
				 	//Open the images and ask user to select threshold and create two ROIs
				 	proceed(dirIm+Imagesfolders[i]+Groupfolders[ii], ImageFiles[iii]);
				 }
		 	}
		 } 
	}
}

//Function that opens an Image and ask the user to select Threshold, cell and center of the cell to be analyzed with the M2 macro
function proceed(dirIm, imagefile){
	//Open image
	run("Bio-Formats Importer", "open=["+dirIm+imagefile+"] autoscale color_mode=Default rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
	name=getTitle();
	//Gaussian filtering
	run("Gaussian Blur 3D...", "x=3 y=3 z=3");
	
	//Get Maximum projection of the image
	run("Z Project...", "projection=[Max Intensity]");
	//Ask the user to create the Cell ROI
	counter=0;
	morecells=true;
	while(morecells==true){
		number=0;
		cont=true;
		while (number!=1 && cont==true){
			//selection of the cell for Sholl analyisis
			selectWindow("MAX_"+name);
			setTool("polygon");
			waitForUser("Please, select the cell and then press T");
			number=roiManager("count");
			if (number!=1){
				cont=getBoolean("Wrong number of ROI, do you want to continue");
			}
		}
		//Ask the user to create a Point selection in the center of the cell for the Sholl analysis
		while (number!=2 && cont==true){
			selectWindow(name);
			run("Select None");
			setTool("Point");
			waitForUser("Please, select the center of the nuclei of the cell and then press T");
			number=roiManager("count");
			if (number!=2){
				cont=getBoolean("Wrong number of ROI, do you want to continue");
			}
			
		}
		//Ask the user to select the threshold
		run("Threshold...");
		waitForUser("Define the Threshold, please");
		//Get threshold
		getThreshold(lower, upper);
		resetThreshold();
		//Save ROIs to the folder M1 ROIs/
		morecells=false;
		if (cont==true){
			morecells=getBoolean("Do you want to analyze another cell from this image?");
			counter++;
			if (File.exists(dirROIs+groupName+"/")==false) File.makeDirectory(dirROIs+groupName+"/");
			if (File.exists(dirROIs+groupName+"/"+expName+"/")==false) File.makeDirectory(dirROIs+groupName+"/"+expName+"/");
			//Rename Rois to contain the string "Cell" or "Center" followed by the Threshold value
			roiManager("Select", 0);
			roiManager("rename", "Cell"+lower);
			roiManager("Select", 1);
			roiManager("rename", "Center"+lower);
			//Save ROIs into the ROIs folder
			roiManager("Deselect");
			roiManager("save", dirROIs+groupName+"/"+expName+"/"+ImaName+"Cell"+counter+".zip");
			roiManager("reset");
		}
	}
	//Move images to the folder M1 Processed images/
	if (counter>0){
			run("Close All");
			if (File.exists(dirProcessed+groupName+"/")==false) File.makeDirectory(dirProcessed+groupName+"/");
			if (File.exists(dirProcessed+groupName+"/"+expName+"/")==false) File.makeDirectory(dirProcessed+groupName+"/"+expName+"/");
			File.rename(dirIm+imagefile, dirProcessed+groupName+"/"+expName+"/"+imagefile);
		}
	//Move images without cells to the folder M1 Not used/
	if (counter==0){
		if (File.exists(dirNotUsed+groupName+"/")==false) File.makeDirectory(dirNotUsed+groupName+"/");
		if (File.exists(dirNotUsed+groupName+"/"+expName+"/")==false) File.makeDirectory(dirNotUsed+groupName+"/"+expName+"/");
		File.rename(dirIm+imagefile, dirNotUsed+groupName+"/"+expName+"/"+imagefile);
	}
}







