/* 
    This file si part of CellShaper.
    
    CellShaper is a pack of ImageJ macros developed 
    to analyze the morphology of hippocampal stem cells,
    Copyright (C) 2019  Jorge Valero Gómez-Lobo.

   CellShaper is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CellShaper is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <https://www.gnu.org/licenses/>.
*/

//This macro has been developed by Dr Jorge Valero (jorge.valero@achucarro.org). 
//If you have any doubt about how to use it, please contact me.

//License
Dialog.create("GNU GPL License");
Dialog.addMessage("CellShaper (C) 2019 Jorge Valero Gomez-Lobo.");
Dialog.setInsets(10, 20, 0);
Dialog.addMessage("CellShaper comes with ABSOLUTELY NO WARRANTY; click on help button for details.");
Dialog.setInsets(0, 20, 0);
Dialog.addMessage("This is free software, and you are welcome to redistribute it under certain conditions; click on help button for details.");
Dialog.addHelp("https://www.gnu.org/licenses/");
Dialog.show();

//Set Batch mode
setBatchMode(true);
run("Colors...", "foreground=white background=black selection=yellow");

//Reset Images and roiManager
run("Close All");
roiManager("reset");

//Create tables for current experiment
	tablearray=newArray("Group", "Experiment", "Image", "RoiName", "Cell type",  "Probability of Omega", "Enclossing radius",  "Sum intersection", "Regression coefficient (log-log)", "Coefficient of determination (log-log)", "Regression coefficient (Semi-log)", "Coefficient of determination (Semi-log)", "Determination ratio (Semi-log/log-log)", "Best fit K", "50 microns Sum intersection", "50 microns Regression coefficient (log-log)", "50 microns Coefficient of determination (log-log)", "50 microns Regression coefficient (Semi-log)", "50 microns Coefficient of determination (Semi-log)", "50 microns Determination ratio (Semi-log/log-log)", "50 microns Best fit K", "Intersections above 50 microns");
	tablecreator("StemSholl Results", tablearray);
	

//Get folders address
dir=getDirectory("Please, select the general directory");
Folders=getFileList(dir);

//Detect and create folders
dirProcessed=dir+"M1 Processed images/";
dirROIs=dir+"M1 ROIs/";
dirResults=dir+"M2 Results/";
if (File.exists(dirResults)==false) File.makeDirectory(dirResults);

//Read folders to analyze images
ROIsfolders=getFileList(dirROIs);
for (i=0; i<ROIsfolders.length; i++){
	//Read ROIs folder
	if (endsWith(ROIsfolders[i], "/")){
		 //Get group folder and name
		 sizefoldName= lengthOf(ROIsfolders[i]);
		 groupName=substring(ROIsfolders[i], 0, sizefoldName-1);
		 Groupfolders=getFileList(dirROIs+ROIsfolders[i]);
		 //Go inside Group folders
		 for (ii=0; ii<Groupfolders.length; ii++){
		 	//Get experiment folder and name
		 	if (endsWith(Groupfolders[ii], "/")){
				sizefoldName= lengthOf(Groupfolders[ii]);
		 		expName=substring(Groupfolders[ii], 0, sizefoldName-1);
				 ROIFiles=getFileList(dirROIs+ROIsfolders[i]+Groupfolders[ii]);
				//Go inside ROIs folder
				 for (iii=0; iii<ROIFiles.length; iii++){
				 	sizefoldName= lengthOf(ROIFiles[iii]);
				 	ROIName=substring(ROIFiles[iii], 0, sizefoldName-4);
				 	Cellend=lastIndexOf(ROIName, "Cell");
				 	FileName=substring(ROIName, 0, Cellend);
				 	CellName=substring(ROIName, Cellend);
				 	//Analyze the ROIs (cells) in the images
				 	proceed(groupName+"/"+expName+"/", ROIFiles[iii]);
				 }
		 	}
		 } 
	}
}
 //Save experiment Table
savetab("StemSholl Results", dir+"M2 Results/");
savetab("Intersections", dir+"M2 Results/");
expdata();
savetab("Mean Intersections", dir+"M2 Results/");
savetab("Mean Alpha Cells Intersections", dir+"M2 Results/");
savetab("Mean Omega Cells Intersections", dir+"M2 Results/");
savetab("Mean StemSholl ", dir+"M2 Results/");
savetab("Mean Omega Cells Sholl ", dir+"M2 Results/");
savetab("Mean Alpha Cells Sholl", dir+"M2 Results/");
savetab("Alpha Cells Intersections", dir+"M2 Results/");
savetab("Omega Cells Intersections", dir+"M2 Results/");
selectWindow("Log");
run("Close");
print("MY JOB IS DONE!");
selectWindow("Log");

				
//Function that analyze an image using its associated ROIs
function proceed(dirIm, imagefile){
	
	//Open image
	imaFileop=dirProcessed+dirIm+FileName+".tif";
	run("Bio-Formats Importer", "open=["+imaFileop+"] autoscale color_mode=Default rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
	
	//Get image parameters
	name=getTitle();
	maxThrs=pow(2,bitDepth());
	getDimensions(width, height, channels, slices, frames);
	getPixelSize(unit, pixelWidth, pixelHeight);
	width=width*pixelWidth;
	//Load Image ROIs
	roiManager("Open", dirROIs+dirIm+imagefile);

	//Filter the image
	run("Gaussian Blur 3D...", "x=3 y=3 z=3");
	//run("Smooth (3D)", "method=Gaussian sigma=0.3 use");

	//Get information of the ROIs
	roiManager("List");

	//Get the channel where Cell ROI was created
	roiManager("Select", 0);
	Channel=infoTab("Overlay Elements", 1, "C");
	if(Channel==0) Channel=parseFloat(infoTab("Overlay Elements", 1, "Z"));
	run("Select None");

	//Duplicate the channel used to create the cell ROI and name the Duplicated image "Chan"
	run("Duplicate...", "title=Chan duplicate channels="+Channel);
	//Close original image
	selectWindow(name);
	run("Close");

	//Clear the area outside Cell ROI
	selectWindow("Chan");
	roiManager("Select", 0);
	run("Clear Outside", "stack");

	//Get information about the Z position of the point ROI and the Threshold (Final number in the name of the ROI)
	roiManager("Select", 1);
	RoiName=infoTab("Overlay Elements", 1, "Name");
	Zpoint=parseFloat(infoTab("Overlay Elements", 2, "Z"));
	Thrs=parseFloat(substring(RoiName, 4));

	//Convert the image to Binary using the threshold previously defined when ROIs were created
	setThreshold(Thrs, maxThrs);

	//Place point ROI in the Binary image in the correz Z position
	roiManager("Select", 1);
	setSlice(Zpoint);
	
	//Run Sholl Analysis
	run("Sholl Analysis...", "starting=0 ending="+width+" radius_step=5 ignore enclosing=1 #_primary=4 fit show linear polynomial=[Best fitting degree] semi-log log-log normalizer=Volume directory=[]");
	transform();
	selectWindow("Chan");
	run("Sholl Analysis...", "starting=0 ending=50 radius_step=5 ignore enclosing=1 #_primary=4 fit show linear polynomial=[Best fitting degree] semi-log log-log normalizer=Volume directory=[]");
	
	//Close not needed windows
	closerer("Sholl profile (Log-log) for Chan");
	closerer("Sholl profile (Linear) for Chan");
	closerer("Chan_Sholl-Profiles");
	closerer("Overlay Elements");
	roiManager("reset");
	run("Close All");

	//Obtain information from the "Sholl Results" table and place it in the results table for this experiment
   if (isOpen("Sholl Results")){
   	 Encls=infoTab("Sholl Results", 1, "Enclosing radius");
	    SumInter=infoTab("Sholl Results", 1, "Sum inters.");
	    SemiCoef=infoTab("Sholl Results", 1, "Regression coefficient (Semi-log)");
	    SemicoefDet=infoTab("Sholl Results", 1, "Regression R^2 (Semi-log)");
	    RegCoef=infoTab("Sholl Results", 1, "Regression coefficient (Log-log)");
	    CoefDet=infoTab("Sholl Results", 1, "Regression R^2 (Log-log)");
	    DetRatio=(parseFloat(SemicoefDet))/(parseFloat(CoefDet));
	    if (DetRatio>1) BestFit= SemiCoef;
	    else BestFit= RegCoef;
	   	SumInter50=infoTab("Sholl Results", 2, "Sum inters.");
	    RegCoef50=infoTab("Sholl Results", 2, "Regression coefficient (Log-log)");
	    CoefDet50=infoTab("Sholl Results",2, "Regression R^2 (Log-log)");
	    SemiCoef50=infoTab("Sholl Results", 2, "Regression coefficient (Semi-log)");
	    SemicoefDet50=infoTab("Sholl Results",2, "Regression R^2 (Semi-log)");
	    DetRatio50=parseFloat(SemicoefDet50)/parseFloat(CoefDet50);
	    if (DetRatio50>1) BestFit50= SemiCoef50;
	    else BestFit50= RegCoef50;
	    rest=parseFloat(SumInter)-parseFloat(SumInter50);

	    //Classification Model
	    ClassVar=153.623+(-0.496*Encls)+(-119.269*CoefDet50);
	    ProbOmega=exp(ClassVar)/(1+exp(ClassVar));
	    if (ClassVar>0) Clase="Omega";
	    else Clase="Alpha";
	    
	    //Print StemSholl results
		tablearray=newArray(groupName, expName, imagefile, RoiName, Clase, ProbOmega,  Encls, SumInter, -RegCoef, CoefDet, -SemiCoef, SemicoefDet, DetRatio, -BestFit, SumInter50, -RegCoef50, CoefDet50, -SemiCoef50, SemicoefDet50, DetRatio50, -BestFit50, rest);
		tableprinter("StemSholl Results", tablearray);
   }
	
	//Eliminate residual information in the memory
	closerer("Sholl Results");
	run("Collect Garbage");
}




//TABLE FUNCTIONS

//Function to create tables with mean data per experiment
function expdata(){
	tablearray=newArray("Group", "Experiment", "%Alpha", "%Omega", "N alpha", "N Omega", "Mean Omega Probability", "Mean enclossing radius", "Mean Sum intersections", "Mean reg coef (log-log)", "Mean Coefficient of determination (log-log)", "Mean reg coef (semi-log)",  "Mean Coefficient of determination (Semi-log)", "Mean det ratio (semi-log/log-log)", "Mean best fit K", "Mean 50 microns Sum inters", "Mean 50 microns Reg coef (log-log)", "Mean 50 microns Reg coef (semi-log)", "Mean 50 microns Det ratio (Semi-log/log-log)", "Mean 50 microns best fit K", "Mean intersection above 50 microns"); 
	tablecreator("Mean StemSholl ", tablearray);
	tablearray=newArray("Group", "Experiment", "%Alpha", "N alpha", "Mean Omega Probability", "Mean enclossing radius", "Mean Sum intersections", "Mean reg coef (log-log)", "Mean Coefficient of determination (log-log)", "Mean reg coef (semi-log)",  "Mean Coefficient of determination (Semi-log)", "Mean det ratio (semi-log/log-log)", "Mean best fit K", "Mean 50 microns Sum inters", "Mean 50 microns Reg coef (log-log)", "Mean 50 microns Reg coef (semi-log)", "Mean 50 microns Det ratio (Semi-log/log-log)", "Mean 50 microns best fit K", "Mean intersection above 50 microns");
	tablecreator("Mean Alpha Cells Sholl", tablearray);
	tablearray=newArray("Group", "Experiment", "%Omega", "N omega", "Mean Omega Probability", "Mean enclossing radius", "Mean Sum intersections", "Mean reg coef (log-log)", "Mean Coefficient of determination (log-log)", "Mean reg coef (semi-log)",  "Mean Coefficient of determination (Semi-log)", "Mean det ratio (semi-log/log-log)", "Mean best fit K", "Mean 50 microns Sum inters", "Mean 50 microns Reg coef (log-log)", "Mean 50 microns Reg coef (semi-log)", "Mean 50 microns Det ratio (Semi-log/log-log)", "Mean 50 microns best fit K", "Mean intersection above 50 microns");
	tablecreator("Mean Omega Cells Sholl ", tablearray);
	selectWindow("StemSholl Results");
	tabinfo=getInfo();
	Ltab=split(tabinfo, "\n");
	Ctab=split(Ltab[0], "\t");
	AlphaCtab=newArray(Ctab.length);
	OmegaCtab=newArray(Ctab.length);
	Ctab=newArray(OmegaCtab.length);
	selectWindow("Intersections");
	tabinfo=getInfo();
	IntLtab=split(tabinfo, "\n");
	IntCtab=split(IntLtab[0], "\t");
	tablecreator("Omega Cells Intersections", IntCtab);
	tablecreator("Alpha Cells Intersections", IntCtab);
	tablearray1=Array.trim(IntCtab, 2);
	tablearray2=Array.slice(IntCtab, 4, IntCtab.length);
	tablearray=Array.concat (tablearray1, tablearray2);
	tablecreator("Mean Intersections", tablearray);
	tablecreator("Mean Alpha Cells Intersections", tablearray);
	tablecreator("Mean Omega Cells Intersections", tablearray);
	AlphaIntCtab=newArray(IntCtab.length);
	OmegaIntCtab=newArray(IntCtab.length);
	IntCtab=newArray(AlphaIntCtab.length);
	Nalpha=0;
	Nomega=0;
	for (i=1; i<Ltab.length; i++){
		Ctabtemp=split(Ltab[i], "\t");
		IntCtabtemp=split(IntLtab[i], "\t");
		type=Ctabtemp[4];
		if (type=="Alpha") tableprinter("Alpha Cells Intersections", IntCtabtemp);
		else tableprinter("Omega Cells Intersections", IntCtabtemp);
		grouptemp=Ctabtemp[0];
		experimentemp=Ctabtemp[1];
		if (i==1){
			group=grouptemp;
			experiment=experimentemp;
		}
		if (grouptemp!=group && experimentemp!=experiment){
				NTot=Nalpha+Nomega;
				ArrSize=Ctab.length;
				IntArrSize=IntCtab.length;
				for (ii=5; ii<Ctab.length; ii++) Ctab[ii]=parseFloat(Ctab[ii])/NTot;
				for (ii=4; ii<IntCtab.length; ii++) IntCtab[ii]=parseFloat(IntCtab[ii])/NTot;
				for (ii=5; ii<Ctab.length; ii++) AlphaCtab[ii]=parseFloat(AlphaCtab[ii])/Nalpha;
				for (ii=4; ii<IntCtab.length; ii++) AlphaIntCtab[ii]=parseFloat(AlphaIntCtab[ii])/Nalpha;
				for (ii=5; ii<Ctab.length; ii++) OmegaCtab[ii]=parseFloat(OmegaCtab[ii])/Nomega;
				for (ii=4; ii<IntCtab.length; ii++) OmegaIntCtab[ii]=parseFloat(OmegaIntCtab[ii])/Nomega;
				PerOmega=(100/NTot)*Nomega;
				PerAlpha=(100/NTot)*Nalpha;
				IntCtab=Array.slice(IntCtab, 2, IntArrSize);
				IntCtab[0]=group;
				IntCtab[1]=experiment;
				AlphaIntCtab=Array.slice(AlphaIntCtab,2, AlphaIntCtab.length);
				AlphaIntCtab[0]=group;
				AlphaIntCtab[1]=experiment;
				OmegaIntCtab=Array.slice(OmegaIntCtab,2, OmegaIntCtab.length);
				OmegaIntCtab[0]=group;
				OmegaIntCtab[1]=experiment;
				tableprinter("Mean Intersections", IntCtab);
				tableprinter("Mean Alpha Cells Intersections", AlphaIntCtab);
				tableprinter("Mean Omega Cells Intersections", OmegaIntCtab);
				extraArr=newArray(1);
				Ctab=Array.concat(extraArr, Ctab);
				Ctab[0]=group;
				Ctab[1]=experiment;
				Ctab[2]=PerAlpha;
				Ctab[3]=PerOmega;
				Ctab[4]=Nalpha;
				Ctab[5]=Nomega;
				tableprinter("Mean StemSholl ", Ctab);
				AlphaCtab=Array.slice(AlphaCtab, 1, AlphaCtab.length);
				AlphaCtab[0]=group;
				AlphaCtab[1]=experiment;
				AlphaCtab[2]=PerAlpha;
				AlphaCtab[3]=Nalpha;
				tableprinter("Mean Alpha Cells Sholl", AlphaCtab);
				OmegaCtab=Array.slice(OmegaCtab, 1, OmegaCtab.length);
				OmegaCtab[0]=group;
				OmegaCtab[1]=experiment;
				OmegaCtab[2]=PerOmega;
				OmegaCtab[3]=Nomega;
				tableprinter("Mean Omega Cells Sholl ", OmegaCtab);
				Nomega=0;
				Nalpha=0;
				group=grouptemp;
				experiment=experimentemp;
				AlphaCtab=newArray(ArrSize);
				OmegaCtab=newArray(ArrSize);
				Ctab=newArray(ArrSize);
				IntCtab=newArray(IntArrSize);
				OmegaIntCtab=newArray(IntArrSize);
				AlphaIntCtab=newArray(IntArrSize);
				for (ii=5; ii<Ctab.length; ii++) Ctab[ii]=Ctabtemp[ii];
				for (ii=4; ii<IntCtab.length; ii++) IntCtab[ii]=IntCtabtemp[ii];
				if (type=="Alpha"){
					Nalpha++;
					for (ii=5; ii<Ctab.length; ii++) AlphaCtab[ii]=Ctabtemp[ii];
					for (ii=4; ii<IntCtab.length; ii++) AlphaIntCtab[ii]=IntCtabtemp[ii];
				}
				if (type=="Omega"){
					Nomega++;
					for (ii=5; ii<Ctab.length; ii++) OmegaCtab[ii]=Ctabtemp[ii];
					for (ii=4; ii<IntCtab.length; ii++) OmegaIntCtab[ii]=IntCtabtemp[ii];
				}
			}	
		else{
			type=Ctabtemp[4];
			for (ii=5; ii<Ctab.length; ii++) Ctab[ii]=parseFloat(Ctab[ii])+parseFloat(Ctabtemp[ii]);
			for (ii=4; ii<IntCtab.length; ii++) IntCtab[ii]=parseFloat(IntCtab[ii])+parseFloat(IntCtabtemp[ii]);
			if (type=="Alpha"){
				Nalpha++;
				for (ii=5; ii<Ctab.length; ii++) AlphaCtab[ii]=parseFloat(AlphaCtab[ii])+parseFloat(Ctabtemp[ii]);
				for (ii=4; ii<IntCtab.length; ii++) AlphaIntCtab[ii]=parseFloat(AlphaIntCtab[ii])+parseFloat(IntCtabtemp[ii]);
			}
			if (type=="Omega"){
				Nomega++;
				for (ii=5; ii<Ctab.length; ii++) OmegaCtab[ii]=parseFloat(OmegaCtab[ii])+parseFloat(Ctabtemp[ii]);
				for (ii=4; ii<IntCtab.length; ii++) OmegaIntCtab[ii]=parseFloat(OmegaIntCtab[ii])+parseFloat(IntCtabtemp[ii]);
			}
		}
	}
	NTot=Nalpha+Nomega;
	for (ii=5; ii<Ctab.length; ii++) Ctab[ii]=parseFloat(Ctab[ii])/NTot;
	for (ii=4; ii<IntCtab.length; ii++) IntCtab[ii]=parseFloat(IntCtab[ii])/NTot;
	for (ii=5; ii<Ctab.length; ii++) AlphaCtab[ii]=parseFloat(AlphaCtab[ii])/Nalpha;
	for (ii=4; ii<IntCtab.length; ii++) AlphaIntCtab[ii]=parseFloat(AlphaIntCtab[ii])/Nalpha;
	for (ii=5; ii<Ctab.length; ii++) OmegaCtab[ii]=parseFloat(OmegaCtab[ii])/Nomega;
	for (ii=4; ii<IntCtab.length; ii++) OmegaIntCtab[ii]=parseFloat(OmegaIntCtab[ii])/Nomega;
	PerOmega=(100/NTot)*Nomega;
	PerAlpha=(100/NTot)*Nalpha;
	IntCtab=Array.slice(IntCtab,2, IntCtab.length);
	IntCtab[0]=group;
	IntCtab[1]=experiment;
	AlphaIntCtab=Array.slice(AlphaIntCtab,2, AlphaIntCtab.length);
	AlphaIntCtab[0]=group;
	AlphaIntCtab[1]=experiment;
	OmegaIntCtab=Array.slice(OmegaIntCtab,2, OmegaIntCtab.length);
	OmegaIntCtab[0]=group;
	OmegaIntCtab[1]=experiment;
	tableprinter("Mean Intersections", IntCtab);
	tableprinter("Mean Alpha Cells Intersections", AlphaIntCtab);
	tableprinter("Mean Omega Cells Intersections", OmegaIntCtab);
	extraArr=newArray(1);
	Ctab=Array.concat(extraArr, Ctab);
	Ctab[0]=group;
	Ctab[1]=experiment;
	Ctab[2]=PerAlpha;
	Ctab[3]=PerOmega;
	Ctab[4]=Nalpha;
	Ctab[5]=Nomega;
	tableprinter("Mean StemSholl ", Ctab);
	AlphaCtab=Array.slice(AlphaCtab, 1, AlphaCtab.length);
	AlphaCtab[0]=group;
	AlphaCtab[1]=experiment;
	AlphaCtab[2]=PerAlpha;
	AlphaCtab[3]=Nalpha;
	tableprinter("Mean Alpha Cells Sholl", AlphaCtab);
	OmegaCtab=Array.slice(OmegaCtab, 1, OmegaCtab.length);
	OmegaCtab[0]=group;
	OmegaCtab[1]=experiment;
	OmegaCtab[2]=PerOmega;
	OmegaCtab[3]=Nomega;
	tableprinter("Mean Omega Cells Sholl ", OmegaCtab);
}
//Function to know the position of a determined heading in a table
function OverlayHeadReader(tableName, headName){
		selectWindow(tableName);
		tableinfo=getInfo();
		Ltab=split(tableinfo, "\n");
		Ctab=split(Ltab[0], "\t");
		for (cvb=0; cvb<Ctab.length; cvb++){
			if (Ctab[cvb]==headName)return(cvb);
		}
} 

//Function that retrieves information from a table
function infoTab(tablename, line, column){
	column=OverlayHeadReader(tablename, column);
	selectWindow(tablename);
	tableinfo=getInfo();
	Ltab=split(tableinfo, "\n");
	Ctab=split(Ltab[line], "\t");
	info=Ctab[column];
	return info;
}

//Function to create tables
function tablecreator(tabname, tablearray){
	
	run("New... ", "name=["+tabname+"] type=Table");
	headings=tablearray[0];
	for (i=1; i<tablearray.length; i++)headings=headings+"\t"+tablearray[i];
	print ("["+tabname+"]", "\\Headings:"+ headings);
	
}

//Function to print information into tables
function tableprinter(tabname, tablearray){
	line=tablearray[0];
	for (i=1; i<tablearray.length; i++) line=line+"\t"+tablearray[i];
	print ("["+tabname+"]", line);
	
}

//Funciton to save tables
function  savetab(tablename, dirRes){
		selectWindow(tablename);
		 saveAs("Text", dirRes+tablename+".xls");
	}

//Function to close windows savetly
function closerer(winName){
	if (isOpen(winName)){
		selectWindow(winName);
		run("Close");
	}
}

//Function to create Intersections table containing the number of intersections per radii of the Sholl analysis
function transform(){
	selectWindow("Chan_Sholl-Profiles");
	tableinfo=getInfo();
	linetable=split(tableinfo, "\n");
	rad=newArray("Group", "Experiment", "Image", "RoiName");
	inters=newArray(groupName, expName, imagefile, RoiName);
	radTemp=newArray(1);
	intersTemp=newArray(1);
	for (t=1; t<linetable.length; t++){
		columntable=split(linetable[t], "\t");
		radTemp[0]=columntable[0];
		intersTemp[0]=columntable[1];
		inters=Array.concat(inters, intersTemp);
		rad=Array.concat(rad, radTemp);
	}	
	if (isOpen("Intersections")==false) {
		tablecreator("Intersections", rad);
		tableprinter("Intersections", inters);
	}
	else {
		selectWindow("Intersections");
		InterInfo=getInfo();
		Interlines=split(InterInfo, "\n");
		Intercolumns=split(Interlines[0], "\t");
		if (Intercolumns.length>=rad.length){
			difLengthtab=Intercolumns.length-rad.length;
			if (difLengthtab>0){
				difArray=newArray(difLengthtab);
				inters=Array.concat(inters, difArray);
			}
		}
		else {
			difLengthtab=linetable.length+4-Intercolumns.length;
			increasetable("Intersections", difLengthtab, rad);
		}
		tableprinter("Intersections", inters);
	}
}

//Function to increase the number of columns of a table
function increasetable(tabname, increase, newheadings){
	difArray=newArray(increase);
	selectWindow(tabname);
	tableinfo=getInfo();
	linetable=split(tableinfo, "\n");
	for (t=0; t<linetable.length; t++){
		if (t==0) tablecreator(tabname+"Increased", newheadings);
		else {
			columntable=split(linetable[t], "\t");
			columntable=Array.concat(columntable, difArray);
			tableprinter(tabname+"Increased", columntable);
		}
	}
	selectWindow(tabname);
	run("Close");
	copytable(tabname+"Increased", tabname);
	closerer(tabname+"Increased");
}

// Function to copy tables
function copytable(oldname, newname){
	first=0;
	if (isOpen(newname)==false) {
		run("New... ", "name=["+newname+"] type=Table");
		first=1;
	}
	selectWindow(oldname);
	tableinfo=getInfo();
	linetable=split(tableinfo, "\n");
	for (t=0; t<linetable.length; t++){
	if (t==0 && first==1) print("["+newname+"]","\\Headings:"+linetable[t]);
	else if (t!=0) print("["+newname+"]",""+linetable[t]);
	}
}